#include "Map.h"

Map::Map()
{

}

Map::~Map()
{

}

int Map::CalculateScore()
{
	return round(boxCount * 10.0f * difficultylevel / (1 + movesCount * 0.001f));
}

void Map::Reset()
{
	end = false;
	drowned = false;
	finishedBoxes = 0;
	movesCount = 0;
	playerRotation = 0;
	playerPosition[0] = playerPositionDefault[0];
	playerPosition[1] = playerPositionDefault[1];
	playerSpriteMovement = 0;

	for (int i = 0; i < boxCount; i++)
	{
		boxPosition[0][i] = boxPositionDefault[0][i];
		boxPosition[1][i] = boxPositionDefault[1][i];
		boxSpriteMovement[i] = 0;
		boxSpriteMovementDirection[i] = 0;
		isDrowned[i] = false;
	}
}

void Map::InitMapSize()
{
	map = new char* [height];
	for (int i = 0; i < height; ++i)
		map[i] = new char[width];

	for (int i = 0; i < 2; i++)
	{
		boxPositionDefault[i] = new int[boxCount];
		boxPosition[i] = new int[boxCount];
		targetPosition[i] = new int[targetCount];
	}

	boxSpriteMovement = new int[boxCount];
	boxSpriteMovementDirection = new int[boxCount];
	isDrowned = new bool[boxCount];
}

bool Map::CheckIfFinished()
{
	finishedBoxes = 0;

	for (int t = 0; t < this->targetCount; t++)
	{

		for (int b = 0; b < this->boxCount; b++)
		{
			if (targetPosition[0][t] == boxPosition[0][b] && targetPosition[1][t] == boxPosition[1][b])
			{
				finishedBoxes++;
			}
		}
	}

	if (finishedBoxes == this->targetCount)
	{
		soundManager.PlayEffect("effect_win");
		this->end = true;
		return true;
	}
	else
	{
		this->end = false;
		return false;
	}

}

void Map::RewriteMapSize(int oldx, int oldy) //wykonywane PO zmianie wartoci zmiennych height, width
{
	char** newmap = new char* [this->height];
	for (int i = 0; i < this->height; ++i)
	{
		newmap[i] = new char[this->width];

		for (int j = 0; j < this->width; j++)
		{
			if (j < oldx && i < oldy)
				newmap[i][j] = this->map[i][j];
			else
				newmap[i][j] = '.';
		}
	}

	this->map = newmap;
}

int Map::MoveHere(int x, int y, char direction, int type, bool isWaterDeadly) // -1 - player // 0-... - block //
{
	if (type == -1)
	{
		x = this->playerPosition[0];
		y = this->playerPosition[1];
		playerRotation = direction;
	}

	int newx = x, newy = y;
	bool occupied = false;

	switch (direction)
	{
	case 0: newy--; break;
	case 1: newx++; break;
	case 2: newy++; break;
	case 3: newx--; break;
	}

	if (newx >= 0 && newy >= 0 && newx < width && newy < height && map[newy][newx] != '.' && map[newy][newx] != '#')
	{
		for (int i = 0; i < boxCount; i++)
		{
			if (boxPosition[0][i] == newx && boxPosition[1][i] == newy)
			{
				if (isDrowned[i])
				{
					occupied = true;
				}
				else
				{
					if (type == -1 && MoveHere(newx, newy, direction, i))
					{
						return 2;
					}

					return 0;
				}
			}
		}

		if (type != -1)
		{
			boxSpriteMovement[type] = 1;
			boxSpriteMovementDirection[type] = direction;
			boxPosition[0][type] = newx;
			boxPosition[1][type] = newy;
			if (map[newy][newx] == 'W' && !occupied)
			{
				isDrowned[type] = true;
				splashed = true;
			}
		}
		else if (map[newy][newx] == 'W' && !occupied) //drowned
		{
			if (isWaterDeadly)
			{
				splashed = true;
				end = true;
				drowned = true;
			}
			else
			{
				return 0;
			}
		}

		return 1;
	}

	return 0;
}